breed [investors investor]
breed [arbitrageurs arbitrageur]
investors-own[out-of-market alpha1 alpha2 buy1 buy2 buyI sell1 sell2 sellI pass1 pass2 passI price1 price2 priceI cash stocks1 stocks2 stocksI subjective-component1 subjective-component2]
arbitrageurs-own[priceArb1 priceArb2 priceArbI out-of-market]
globals [logB1 logS1 logB2 logS2 logBI logSI exePrice1 exePrice2 exePriceI nshares1 nshares2 nsharesI w1 w2 realIndex realStock1 realStock2 list-cash] 

to setup

  clear-all
  set exePrice1 1000
  set exePrice2 900
  set nshares1 100
  set nshares2 200
  set nsharesI 120
  set logB1 []
  set logS1 []
  set logB2 []
  set logS2 []
  set logBI []
  set logSI []
  set realIndex []
  set realStock1 []
  set realStock2 []
  set list-cash []
  reset-ticks
  
  set w1 (exePrice1 * nshares1) / (exePrice1 * nshares1 + exePrice2 * nshares2)
  set w2 (exePrice2 * nshares2) / (exePrice1 * nshares1 + exePrice2 * nshares2)
  
  set exePriceI (w1 * exePrice1 + w2 * exePrice2)
  
  create-investors ninvestors
  
  create-arbitrageurs 1

  let side sqrt ninvestors
  
  let step max-pxcor / side
  
ask investors
  [
    set shape "person"
    set color white
    set out-of-market False
    set size 2
    set stocks1 0
    set stocks2 0
    set stocksI 0  
    set cash 0
  ]
  
    
let an 0

let x 0
let y 0
while [an < ninvestors]
  [
    if x > (side - 1) * step
              [set y y + step
               set x 0       ]
    ask investor an
     [setxy x y]
    set x x + step
    set an an + 1            
  ]
  
  read-file 

end

to go
  
if ticks >= 101 [stop]  

 ;;rational agents use their new information to choose if buying or selling stocks (and the index): if alpha is positive they'll buy the stock, while if it's negative they'll sell the stock
 ;;moreover agents choose to buy or sell the index according to the 'weighted alpha', which is w1*alpha1 + w2*alpha2: if this value is positive they'll buy the index, otherwise they'll sell it 
 ask investors
  [ 
    
    
    ifelse out-of-market [set color white]
    [
     ;; let's generate the two stocks' alphas: investor have received new information from their analyst
     set alpha1 random-normal mean-information asymmetry-degree
     set alpha2 random-normal mean-information asymmetry-degree
      
     ;;choice on the first security 
     ifelse abs alpha1 < information-threshold
     [ 
      ifelse alpha1 > 0 
       [set buy1 True set sell1 False set pass1 False]
       [set buy1 False set sell1 True set pass1 False]]
      [set pass1 True set buy1 False set sell1 False]
       
     if buy1    [set color 15]
     if sell1   [set color 55]
     if pass1   [set color 95]
     
     ;;choice on the second security
     ifelse abs alpha2 < information-threshold
      [
      ifelse alpha2 > 0 
       [set buy2 True set sell2 False set pass2 False]
       [set buy2 False set sell2 True set pass2 False]]
      [set pass2 True set buy2 False set sell2 False]
     
       
     if buy2    [set color 25]
     if sell2   [set color 65]
     if pass2   [set color 105]
     
     ;;choice on the index
     ifelse abs alpha1 < information-threshold and abs alpha2 < information-threshold
      [
       ifelse w1 * alpha1 + w2 * alpha2 > 0
       [set buyI True set sellI False set passI False]
       [set buyI False set sellI True set passI False]]
      [set passI True set buyI False set sellI False]
     
     
     ;;investors use alpha values to fix their prices expectations about the two stocks and the index
     
     if not pass1
     [set subjective-component1 random-normal 0 (asymmetry-degree / 2) 
      ifelse alpha1 > 0 [set price1 exePrice1 + ( exePrice1 * alpha1) / 100 + subjective-component1]
                        [set price1 exePrice1 - ( exePrice1 * alpha1) / 100 + subjective-component1]]
                       
     if not pass2
     [set subjective-component2 random-normal 0 (asymmetry-degree / 2)
      ifelse alpha2 > 0 [set price2 exePrice2 + ( exePrice2 * alpha2 ) / 100 + subjective-component2]
                        [set price2 exePrice2 - ( exePrice2 * alpha2 ) / 100 + subjective-component2]]
     
     if not passI
     [
      ifelse alpha1 > 0 [ifelse alpha2 > 0 
                         [set priceI exePriceI + ( w1 * (( exePrice1 * alpha1) / 100) + w1 * subjective-component1 + w2 * (( exePrice2 * alpha2) / 100) + w2 * subjective-component2 )]
                         [set priceI exePriceI + ( w1 * (( exePrice1 * alpha1) / 100) + w1 * subjective-component1 - w2 * (( exePrice2 * alpha2) / 100) + w2 * subjective-component2 )]]
                        [ifelse alpha2 > 0 
                         [set priceI exePriceI - ( w1 * (( exePrice1 * alpha1) / 100) + w1 * subjective-component1 + w2 * (( exePrice2 * alpha2) / 100) + w2 * subjective-component2 )]
                         [set priceI exePriceI - ( w1 * (( exePrice1 * alpha1) / 100) + w1 * subjective-component1 - w2 * (( exePrice2 * alpha2) / 100) + w2 * subjective-component2 )]
                        ] 
     ]
                        
     
     if (abs alpha1 < information-threshold or abs alpha2 < information-threshold)                
      [set cash cash - cost-of-information]
     
     ] 
    
  ]


;;use these clearing operations if a 'go cycle' is 'a day'
set logB1 []
set logS1 []
set logB2 []
set logS2 []
set logBI []
set logSI []  
    
  
ask investors
  [if not out-of-market
     [
       
     ;;let'create the temporary vector with the price of the first security      
       let tmp1[]
       set tmp1 lput price1 tmp1
       set tmp1 lput who tmp1

       if buy1 [set logB1 lput tmp1 logB1]
       set logB1 reverse sort-by [item 0 ?1 < item 0 ?2] logB1
       
       if sell1 [set logS1 lput tmp1 logS1]
       set logS1 sort-by [item 0 ?1 < item 0 ?2] logS1
       
       ;show logB1
       ;show logS1
       
       ifelse exePrice1 < item ticks realStock1  
            [ask arbitrageurs 
               [set priceArb1 exePrice1 + random-float ( item ticks realStock1 - exePrice1)
                let tmparb1[]
                set tmparb1 lput priceArb1 tmparb1
                set tmparb1 lput who tmparb1
                set logB1 fput tmparb1 logB1
               
              ]
              set logB1 reverse sort-by [item 0 ?1 < item 0 ?2] logB1
             ]
                                                
            [ask arbitrageurs
               [set priceArb1 item ticks realStock1 + random-float (exePrice1 - item ticks realStock1)
                let tmparb1[]
                set tmparb1 lput priceArb1 tmparb1
                set tmparb1 lput who tmparb1
                set logS1 fput tmparb1 logS1

              ] 
               set logS1 sort-by [item 0 ?1 < item 0 ?2] logS1
             ]
            
        ;show logB1
        ;show logS1 
        
        
       ;the arbitrageur buys or sells the stock1 in order to allineate the weighted sum of the two stocks with the current value of the index
       if random-float 1 < arbitrageLevel
        [ifelse w1 * exeprice1 + w2 * exePrice2 < exepriceI 
              [ask arbitrageurs
                 [set priceArb1 exePrice1 + random-float ( (exePriceI - w1 * exeprice1 - w2 * exePrice2)/( w1 + w2 ) ) 
                  let tmparb1[]
                  set tmparb1 lput priceArb1 tmparb1
                  set tmparb1 lput who tmparb1
                  set logB1 fput tmparb1 logB1
                              
                ]
                set logB1 reverse sort-by [item 0 ?1 < item 0 ?2] logB1
              ]
                                                
              [ask arbitrageurs
                 [set priceArb1 exePrice1 + random-float ( (exePriceI - w1 * exeprice1 - w2 * exePrice2)/( w1 + w2 ) )
                  let tmparb1[]
                  set tmparb1 lput priceArb1 tmparb1
                  set tmparb1 lput who tmparb1
                  set logS1 fput tmparb1 logS1
                              

                ] 
                 set logS1 sort-by [item 0 ?1 < item 0 ?2] logS1
               ]
         ]
              
       ;show logB1
       ;show logS1 
            
       
     ;;let'create the temporary vector with the price of the second security
       let tmp2[]
       set tmp2 lput price2 tmp2
       set tmp2 lput who tmp2
       
       if buy2 [set logB2 lput tmp2 logB2]
       set logB2 reverse sort-by [item 0 ?1 < item 0 ?2] logB2
       
       if sell2 [set logS2 lput tmp2 logS2]
       set logS2 sort-by [item 0 ?1 < item 0 ?2] logS2
       
       ;show logB2
       ;show logS2
       
       ifelse exePrice2 < item ticks realStock2  
              [ask arbitrageurs
                 [set priceArb2 exePrice2 + random-float ( item ticks realStock2 - exePrice2)
                  let tmparb2[]
                  set tmparb2 lput priceArb2 tmparb2
                  set tmparb2 lput who tmparb2
                  set logB2 fput tmparb2 logB2
                              
                ]
                set logB2 reverse sort-by [item 0 ?1 < item 0 ?2] logB2
              ]
                                                
              [ask arbitrageurs
                 [set priceArb2 item ticks realStock2 + random-float (exePrice2 - item ticks realStock2)
                  let tmparb2[]
                  set tmparb2 lput priceArb2 tmparb2
                  set tmparb2 lput who tmparb2
                  set logS2 fput tmparb2 logS2
                              

                ] 
                 set logS2 sort-by [item 0 ?1 < item 0 ?2] logS2
               ]
              
         ;show logB2
         ;show logS2
         
         
       ;the arbitrageur buys or sells the stock2 in order to allineate the weighted sum of the two stocks with the current value of the index
       if random-float 1 < arbitrageLevel
        [ifelse w1 * exeprice1 + w2 * exePrice2 < exepriceI 
              [ask arbitrageurs
                 [set priceArb2 exePrice2 + random-float ( (exePriceI - w1 * exeprice1 - w2 * exePrice2)/( w1 + w2 ) ) 
                  let tmparb2[]
                  set tmparb2 lput priceArb2 tmparb2
                  set tmparb2 lput who tmparb2
                  set logB2 fput tmparb2 logB2
                              
                ]
                set logB2 reverse sort-by [item 0 ?1 < item 0 ?2] logB2
              ]
                                                
              [ask arbitrageurs
                 [set priceArb2 exePrice2 + random-float ( (exePriceI - w1 * exeprice1 - w2 * exePrice2)/( w1 + w2 ) )
                  let tmparb2[]
                  set tmparb2 lput priceArb2 tmparb2
                  set tmparb2 lput who tmparb2
                  set logS2 fput tmparb2 logS2
                              

                ] 
                 set logS2 sort-by [item 0 ?1 < item 0 ?2] logS2
               ]
        ]  
              
         ;show logB2
         ;show logS2    
                                           
     
     ;;let'create the temporary vector with the price of the index  
       let tmpI[]
       set tmpI lput priceI tmpI
       set tmpI lput who tmpI
       
       if buyI [set logBI lput tmpI logBI]
       set logBI reverse sort-by [item 0 ?1 < item 0 ?2] logBI
       
       if sellI [set logSI lput tmpI logSI]
       set logSI sort-by [item 0 ?1 < item 0 ?2] logSI
       
       ;show logBI
       ;show logSI
       
       ifelse exePriceI < item ticks realIndex  
             [ask arbitrageurs
                [set priceArbI exePriceI + random-float ( item ticks realIndex - exePriceI)
                 let tmparbI[]
                 set tmparbI lput priceArbI tmparbI
                 set tmparbI lput who tmparbI
                 set logBI fput tmparbI logBI

                ] 
               set logBI reverse sort-by [item 0 ?1 < item 0 ?2] logBI
             ]
                                                
              [ask arbitrageurs
                [set priceArbI item ticks realIndex + random-float (exePriceI - item ticks realIndex)
                 let tmparbI[]
                 set tmparbI lput priceArbI tmparbI
                 set tmparbI lput who tmparbI
                 set logSI fput tmparbI logSI
                              
               ] 
                set logSI sort-by [item 0 ?1 < item 0 ?2] logSI
              ]
              
        ;show logBI
        ;show logSI     
                                                
       
       
   ;;BOOK FOR THE INDEX IN ORDER TO ALLINEATE THE ENDOGENOUS VALUE TO THE EXOGENOUS ONE
   if (not empty? logBI and not empty? logSI and item 0 (item 0 logBI) >= item 0 (item 0 logSI) )
                    [if buyI [ifelse item 1 (item 0 logBI) = who [set exePriceI item 0 (item 0 logSI)
                                                                  let agBI item 1 (item 0 logBI)
                                                                  ask investor agBI [set stocksI stocksI + 1 
                                                                                      set cash cash - exePriceI]
                                                                  if item 1 (item 0 logSI) < ninvestors [let agSI item 1 (item 0 logSI)
                                                                                                                ask investor agSI [set stocksI stocksI - 1 
                                                                                                                set cash cash + exePriceI] ]   ]
                                                                  [if item 1 (item 0 logBI) < ninvestors [set exePriceI item 0 (item 0 logBI)
                                                                                                          let agBI item 1 (item 0 logBI)
                                                                                                          ask investor agBI [set stocksI stocksI + 1 
                                                                                                                   set cash cash - exePriceI] ] 
                                                                   if item 1 (item 0 logSI) < ninvestors [set exePriceI item 0 (item 0 logSI)
                                                                                                          let agSI item 1 (item 0 logSI)
                                                                                                          ask investor agSI [set stocksI stocksI - 1 
                                                                                                                set cash cash + exePriceI] ]    ]
                                                                   ]
                     if sellI [ifelse item 1 (item 0 logSI) = who [set exePriceI item 0 (item 0 logBI)
                                                                   let agSI item 1 (item 0 logSI)
                                                                   ask investor agSI [set stocksI stocksI - 1 
                                                                                      set cash cash + exePriceI]
                                                                   if item 1 (item 0 logBI) < ninvestors [let agBI item 1 (item 0 logBI)
                                                                                                                ask investor agBI [set stocksI stocksI + 1 
                                                                                                                set cash cash - exePriceI] ]   ]
                                                                   [if item 1 (item 0 logSI) < ninvestors [set exePriceI item 0 (item 0 logSI)
                                                                                                           let agSI item 1 (item 0 logSI)
                                                                                                           ask investor agSI [set stocksI stocksI - 1 
                                                                                                                set cash cash + exePriceI] ] 
                                                                   if item 1 (item 0 logBI) < ninvestors [set exePriceI item 0 (item 0 logBI)
                                                                                                          let agBI item 1 (item 0 logBI)
                                                                                                          ask investor agBI [set stocksI stocksI + 1 
                                                                                                                set cash cash - exePriceI] ]    ]
                                                                   ]
                   
                       set logBI but-first logBI
                       set logSI but-first logSI      
                      ]
                    
                    
     ;;BOOK FOR THE FIRST SECURITY IN ORDER TO ALLINEATE THE ENDOGENOUS VALUE TO THE EXOGENOUS ONE
     if (not empty? logB1 and not empty? logS1 and item 0 (item 0 logB1) >= item 0 (item 0 logS1) )
                    [if buy1 [ifelse item 1 (item 0 logB1) = who [set exePrice1 item 0 (item 0 logS1)
                                                                  let agB1 item 1 (item 0 logB1)
                                                                  ask investor agB1 [set stocks1 stocks1 + 1 
                                                                                      set cash cash - exePrice1]
                                                                  if item 1 (item 0 logS1) < ninvestors [let agS1 item 1 (item 0 logS1)
                                                                                                                ask investor agS1 [set stocks1 stocks1 - 1 
                                                                                                                set cash cash + exePrice1] ]   ]
                                                                  [if item 1 (item 0 logB1) < ninvestors [set exePrice1 item 0 (item 0 logB1)
                                                                                                          let agB1 item 1 (item 0 logB1)
                                                                                                          ask investor agB1 [set stocks1 stocks1 + 1 
                                                                                                                set cash cash - exePrice1] ] 
                                                                   if item 1 (item 0 logS1) < ninvestors [set exePrice1 item 0 (item 0 logS1)
                                                                                                          let agS1 item 1 (item 0 logS1)
                                                                                                          ask investor agS1 [set stocks1 stocks1 - 1 
                                                                                                                set cash cash + exePrice1] ]    ]
                                                                   ]
                     if sell1 [ifelse item 1 (item 0 logS1) = who [set exePrice1 item 0 (item 0 logB1)
                                                                   let agS1 item 1 (item 0 logS1)
                                                                   ask investor agS1 [set stocks1 stocks1 - 1 
                                                                                      set cash cash + exePrice1]
                                                                   if item 1 (item 0 logB1) < ninvestors [let agB1 item 1 (item 0 logB1)
                                                                                                                ask investor agB1 [set stocks1 stocks1 + 1 
                                                                                                                set cash cash - exePrice1] ]   ]
                                                                   [if item 1 (item 0 logS1) < ninvestors [set exePrice1 item 0 (item 0 logS1)
                                                                                                           let agS1 item 1 (item 0 logS1)
                                                                                                           ask investor agS1 [set stocks1 stocks1 - 1 
                                                                                                                set cash cash + exePrice1] ] 
                                                                   if item 1 (item 0 logB1) < ninvestors [set exePrice1 item 0 (item 0 logB1)
                                                                                                          let agB1 item 1 (item 0 logB1)
                                                                                                          ask investor agB1 [set stocks1 stocks1 + 1 
                                                                                                                set cash cash - exePrice1] ]    ]
                                                                   ] 
                       
                       set logB1 but-first logB1
                       set logS1 but-first logS1     
                      ]  
                    
      ;;BOOK FOR THE SECOND SECURITY IN ORDER TO ALLINEATE THE ENDOGENOUS VALUE TO THE EXOGENOUS ONE
      if (not empty? logB2 and not empty? logS2 and item 0 (item 0 logB2) >= item 0 (item 0 logS2) )
                    [if buy2 [ifelse item 1 (item 0 logB2) = who [set exePrice2 item 0 (item 0 logS2)
                                                                  let agB2 item 1 (item 0 logB2)
                                                                  ask investor agB2 [set stocks2 stocks2 + 1 
                                                                                      set cash cash - exePrice2]
                                                                  if item 1 (item 0 logS2) < ninvestors [let agS2 item 1 (item 0 logS2)
                                                                                                                ask investor agS2 [set stocks2 stocks2 - 1 
                                                                                                                set cash cash + exePrice2] ]   ]
                                                                  [if item 1 (item 0 logB2) < ninvestors [set exePrice2 item 0 (item 0 logB2)
                                                                                                          let agB2 item 1 (item 0 logB2)
                                                                                                          ask investor agB2 [set stocks2 stocks2 + 1 
                                                                                                                set cash cash - exePrice2] ] 
                                                                   if item 1 (item 0 logS2) < ninvestors [set exePrice2 item 0 (item 0 logS2)
                                                                                                          let agS2 item 1 (item 0 logS2)
                                                                                                          ask investor agS2 [set stocks2 stocks2 - 1 
                                                                                                                set cash cash + exePrice2] ]    ]
                                                                   ]
                     if sell2 [ifelse item 1 (item 0 logS2) = who [set exePrice2 item 0 (item 0 logB2)
                                                                   let agS2 item 1 (item 0 logS2)
                                                                   ask investor agS2 [set stocks2 stocks2 - 1 
                                                                                      set cash cash + exePrice2]
                                                                   if item 1 (item 0 logB2) < ninvestors [let agB2 item 1 (item 0 logB2)
                                                                                                                ask investor agB2 [set stocks2 stocks2 + 1 
                                                                                                                set cash cash - exePrice2] ]   ]
                                                                   [if item 1 (item 0 logS2) < ninvestors [set exePrice2 item 0 (item 0 logS2)
                                                                                                           let agS2 item 1 (item 0 logS2)
                                                                                                           ask investor agS2 [set stocks2 stocks2 - 1 
                                                                                                                set cash cash + exePrice2] ] 
                                                                   if item 1 (item 0 logB2) < ninvestors [set exePrice2 item 0 (item 0 logB2)
                                                                                                          let agB2 item 1 (item 0 logB2)
                                                                                                          ask investor agB2 [set stocks2 stocks2 + 1 
                                                                                                                set cash cash - exePrice2] ]    ]
                                                                   ] 
                       
                       set logB2 but-first logB2
                       set logS2 but-first logS2     
                      ]
                    
                                           
         ;print "logB1 = " show logB1
         ;print "logS1 = " show logS1
         ;print "logB2 = " show logB2
         ;print "logS2 = " show logS2
         ;print "logBI = " show logBI
         ;print "logSI = " show logSI      
                    
         
         ]            

     
   if random-float 1 < out-of-marketLevel
       [if (exePrice1 < 500 and exePrice2 < 500 and exepriceI < 500) [set out-of-market True] ]
       
  ]

if ticks = 100 [ask investors
                  [set list-cash lput cash list-cash]
                ]

tick
;show ticks
;in future implementation it would be interesting to make empty both the side of 
;the market at regular intervals of more than one cycle     

    
graph
     
end

to read-file
    
    file-open "Index_values_format.txt"
     while [not file-at-end?][
       ;read the line 
        let in file-read
        set realIndex lput in realIndex
        ]
    file-close
    
    file-open "Stock1_values_format.txt"
     while [not file-at-end?][
       ;read the line 
        let in file-read
        set realStock1 lput in realStock1
        ]
    file-close
    
    file-open "Stock2_values_format.txt"
     while [not file-at-end?][
       ;read the line 
        let in file-read
        set realStock2 lput in realStock2
        ]
    file-close
    
end

to graph
 
set-current-plot "exePrice1"
plot exePrice1
set-current-plot "exePrice2"
plot exePrice2
set-current-plot "exePriceI"
plot exePriceI

end     
@#$#@#$#@
GRAPHICS-WINDOW
210
10
649
470
-1
-1
13.0
1
10
1
1
1
0
1
1
1
0
32
0
32
0
0
1
ticks
30.0

BUTTON
14
22
80
55
NIL
setup
NIL
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

BUTTON
121
26
184
59
NIL
go
T
1
T
OBSERVER
NIL
NIL
NIL
NIL
1

SLIDER
15
80
187
113
ninvestors
ninvestors
0
300
271
1
1
NIL
HORIZONTAL

SLIDER
15
134
187
167
mean-information
mean-information
-10
10
0
1
1
NIL
HORIZONTAL

SLIDER
16
184
190
217
asymmetry-degree
asymmetry-degree
0
10
1.1
0.1
1
NIL
HORIZONTAL

SLIDER
16
231
201
264
information-threshold
information-threshold
0
10
10
1
1
NIL
HORIZONTAL

SLIDER
18
278
197
311
cost-of-information
cost-of-information
0
2
1.9
0.1
1
NIL
HORIZONTAL

SLIDER
18
332
190
365
out-of-marketLevel
out-of-marketLevel
0
1
1
0.1
1
NIL
HORIZONTAL

PLOT
689
47
889
197
exePrice1
NIL
NIL
0.0
200.0
0.0
1500.0
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" ""

PLOT
694
240
894
390
exePrice2
NIL
NIL
0.0
200.0
0.0
1500.0
true
false
"" ""
PENS
"default" 1.0 0 -16777216 true "" ""

PLOT
982
158
1182
308
exePriceI
NIL
NIL
0.0
200.0
0.0
1500.0
true
false
"" ""
PENS
"default" 1.0 0 -2674135 true "" ""

SLIDER
19
387
191
420
arbitrageLevel
arbitrageLevel
0
1
1
0.1
1
NIL
HORIZONTAL

@#$#@#$#@
## WHAT IS IT?

The model shows the prices dynamics of a three-asset stock market, in which there are two different stocks and an index (which is defined as the weighted sum of the stocks). Investors make a buying or selling price for each security according to the information acquired from their analyst and the auction mechanism rules the price formation. The intervention of an arbitrageur makes the market rational, preventing (almost completely) every arbitrage operation for the investors. The goal of the simulation is to show the dynamics of a market in which agents acquire information about the assets, so performing an active strategy.

## HOW IT WORKS

In the first part of the program investors gather information at each step about the two securities (paying their analyst): this information is represented through an alpha parameter for both stocks, where alphas are values which describe the mispricing (in terms of percentage) of the stock price according to the analyst; the alpha parameter for the index is calculated as weighted sum of the two alphas. If alpha is positive then the asset is underpriced and the investor has to buy it; if alpha is negative the asset is overpriced and the investor will sell it. Alpha parameters are taken randomly according to a normal distribution with standard deviation represents the degree of informational asymmetry in the market (i.e. the different views of the analysts about the market). Every investor uses the alpha values so obtained to compute their buying or selling price, including a subjective and irrational component too.

In the second part of the program agents interact with each other in the auction mechanism. Vectors logB and logS are respectively filled with buying and selling price of every investor. Moreover the arbitrageur, when an investor makes a buying or selling price for a security, proposes two different prices following two different arbitrage strategies:

 1) he looks at the external data series of the corresponding security (created in           Mathematica) and he compares the exogenous price at that istant of time (tick in the     simulation) with the current price in the simulation: if the last one is less then       the exogenous one, he will buy the security, otherwise he will sell it, and the          buying or selling price is always a random between the exogenous price and the           actual one in the simulation. In this way all three asset prices are aligned to          some external data, which can be built upon every kind of financial model.

 2) he looks at the three asset prices in the simulation checking the validity, at 
    each tick, of the no-arbitrage condition: the weighted sum of the two stock prices       has to be equal to the index price. If this price is greater than the weighted           sum, the arbitrageur will buy the two stocks proposing two prices such to restore        the equilibrium condition and to apply the same price changes to both assets; if the     index price is lower than the weighted sum of the stock prices the arbitrageur will      sell the stocks, proposing again two prices such to restore the equilibrium              condition and to apply the same price changes to both assets. With this operation        the three markets (related to each asset) are linked together and the                    operations an investor can make on one asset can influence the price of another one.

So the arbitrageur interacts with the investors in the auction market for every asset. While we count the cash and the number of stocks owned by every investor, we chose to not count such variables for the arbitrageur, so simplifying the trades.

Every time we have a possible matching in the book of some asset (the buying price is greater or equal than the selling price) the trade is concluded: we can have trading between two investors, between an investor and the arbitrageur and between two arbitrageurs. 

If the users wants to control, step by step, the auction/trading mechanism she can do that using the strings of code with the command show, in order to visualize how the vectors logB and logS (which are actually matrixes with the second column collecting the who numbers of the investors) change over over time.



## HOW TO USE IT

1. Adjust the slider parameters (see below).
2. Press the SETUP button.
3. Press the GO button to begin the simulation.
4. Look at the monitor to see the current trades in the market: different colors show      if an investor is willing to buy or sell an asset (white color means the investor is     out of market).
5. Look at the exePrice plots to watch the asset prices dynamics.

Parameters:
NINVESTORS: The number of investors in the market.
MEAN-INFORMATION: The mean of the normal distribution of alpha parameters.
ASYMMETRY-DEGREE: The standard deviation of the normal distribution of alpha                               parameters: it quantifies the informational asymmetry in the market.
INFORMATION-THRESHOLD: The threshold of reliability of alpha parameters: if alpha is                            greater than the threshold the investor does not rely on the                             analyst and prefers to not partecipate to the auction (the                               boolean variable pass is set true and the investor becomes blue).
COST-OF-INFORMATION: The cash payed from an investor to her anaylst if she has gathered                       a reliable alpha parameter (i.e. under the information-threshold).
OUT-OF-MARKETLEVEL: The probability of testing the bankruptcy condition for the assets.                      If a random between 0 and 1 is lower than the out-of-marketLevel we                      test if all asset prices are under a common level (here 500): if                         this is true the investor chooses to exit from the market, setting                       the out-of-market variable as true.
ARBITRAGELEVEL: The probability with which the arbitrageur operates on the market to                     restore the no-arbitrage condition: an arbitrageLevel equal to 1 means                   the arbitrageur acts every time, an arbitrageLevel of 0 corresponds to                   the total absence of arbitrageur's intervention.


## THINGS TO NOTICE

Remember that the program works reading three data files for the assets, which have to be saved in the same directory where the program is. Data files are created for a certain number of time units (ticks in the simulation): the program stops after this number of ticks.

In the program we created only one arbitrageur, calling her actions every time it needs, but in reality we have different arbitrageurs who propose different buying or selling prices, pursuing the same strategy. We assume that the only arbitrageur in the simulation represents all the possible arbitrageurs operating in the market.

WARNING!: the program is built in order to distinguish an investor from the arbitrageur through the who number: the arbitrageur is identified with a who number corresponding to nivestors plus one (indeed the who numbers are assigned irrespective of breeds). So, if during the simulation we change the parameter ninvestors, the program shows an error because it can no more efficiently distinguish the arbitrageur from any other investor. Then we have to keep constant the number of investors during the simulation!

Notice that if asymmetry-degree is zero (the so-called homogeneous expectations in the market) asset prices remain substantially constant during the simulation: informational asymmetry is the driving force in market dynamics. If all investors have the same information about assets, prices are in equilibrium.

## THINGS TO TRY

Using Mathematica Link to NetLogo you can load our model on a Mathematica notebook checking the efficacy of the arbitrage operations. In particular we can compare the exogenous index data with the weighted sum of the stock prices, verifying, under different parameters settings, the resilience of the no-arbitrage condition.

The program allows agents to borrow cash in order to continue to invest in the market. We can easily add a debt constraint: if cash becomes lower than all of the current asset prices, the investor can no more participate to the market and its variable out-of-market is set true. With this further constraint the parameter cost-of-information becomes important in the asset prices dynamics and we can explore its role in the simulation.

Vary the asymmetry-degree to observe different patterns in the market, with different levels of informational asymmetry. What is the effect of informational asymmetry on the arbitrage efficacy (market rationality)?
 

## EXTENDING THE MODEL

You can modify the code in order to show or print (at each tick or simply at the end of the simulation) the cash and the number stocks owned by every investor. So you can compare investors' profits at the end of the simulation under different values of asymmetry-degree.

Try to modify the code taking into account cash and stocks owned by the arbitrageur: it can be interesting to test the earnings from arbitrage operations, under different parameters settings. 

Finally you could create more arbitrageurs, each identified with a particular who number, counting their cash and their stocks. Competiton among different arbitrageurs is another theme to explore in a complex system as a financial market.

## NETLOGO FEATURES

(interesting or unusual features of NetLogo that the model uses, particularly in the Code tab; or where workarounds were needed for missing features)

## RELATED MODELS

(models in the NetLogo Models Library and elsewhere which are of related interest)

## CREDITS AND REFERENCES

(a reference to the model's URL on the web if it has one, as well as any other necessary credits, citations, and links)
@#$#@#$#@
default
true
0
Polygon -7500403 true true 150 5 40 250 150 205 260 250

airplane
true
0
Polygon -7500403 true true 150 0 135 15 120 60 120 105 15 165 15 195 120 180 135 240 105 270 120 285 150 270 180 285 210 270 165 240 180 180 285 195 285 165 180 105 180 60 165 15

arrow
true
0
Polygon -7500403 true true 150 0 0 150 105 150 105 293 195 293 195 150 300 150

box
false
0
Polygon -7500403 true true 150 285 285 225 285 75 150 135
Polygon -7500403 true true 150 135 15 75 150 15 285 75
Polygon -7500403 true true 15 75 15 225 150 285 150 135
Line -16777216 false 150 285 150 135
Line -16777216 false 150 135 15 75
Line -16777216 false 150 135 285 75

bug
true
0
Circle -7500403 true true 96 182 108
Circle -7500403 true true 110 127 80
Circle -7500403 true true 110 75 80
Line -7500403 true 150 100 80 30
Line -7500403 true 150 100 220 30

butterfly
true
0
Polygon -7500403 true true 150 165 209 199 225 225 225 255 195 270 165 255 150 240
Polygon -7500403 true true 150 165 89 198 75 225 75 255 105 270 135 255 150 240
Polygon -7500403 true true 139 148 100 105 55 90 25 90 10 105 10 135 25 180 40 195 85 194 139 163
Polygon -7500403 true true 162 150 200 105 245 90 275 90 290 105 290 135 275 180 260 195 215 195 162 165
Polygon -16777216 true false 150 255 135 225 120 150 135 120 150 105 165 120 180 150 165 225
Circle -16777216 true false 135 90 30
Line -16777216 false 150 105 195 60
Line -16777216 false 150 105 105 60

car
false
0
Polygon -7500403 true true 300 180 279 164 261 144 240 135 226 132 213 106 203 84 185 63 159 50 135 50 75 60 0 150 0 165 0 225 300 225 300 180
Circle -16777216 true false 180 180 90
Circle -16777216 true false 30 180 90
Polygon -16777216 true false 162 80 132 78 134 135 209 135 194 105 189 96 180 89
Circle -7500403 true true 47 195 58
Circle -7500403 true true 195 195 58

circle
false
0
Circle -7500403 true true 0 0 300

circle 2
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240

cow
false
0
Polygon -7500403 true true 200 193 197 249 179 249 177 196 166 187 140 189 93 191 78 179 72 211 49 209 48 181 37 149 25 120 25 89 45 72 103 84 179 75 198 76 252 64 272 81 293 103 285 121 255 121 242 118 224 167
Polygon -7500403 true true 73 210 86 251 62 249 48 208
Polygon -7500403 true true 25 114 16 195 9 204 23 213 25 200 39 123

cylinder
false
0
Circle -7500403 true true 0 0 300

dot
false
0
Circle -7500403 true true 90 90 120

face happy
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 255 90 239 62 213 47 191 67 179 90 203 109 218 150 225 192 218 210 203 227 181 251 194 236 217 212 240

face neutral
false
0
Circle -7500403 true true 8 7 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Rectangle -16777216 true false 60 195 240 225

face sad
false
0
Circle -7500403 true true 8 8 285
Circle -16777216 true false 60 75 60
Circle -16777216 true false 180 75 60
Polygon -16777216 true false 150 168 90 184 62 210 47 232 67 244 90 220 109 205 150 198 192 205 210 220 227 242 251 229 236 206 212 183

fish
false
0
Polygon -1 true false 44 131 21 87 15 86 0 120 15 150 0 180 13 214 20 212 45 166
Polygon -1 true false 135 195 119 235 95 218 76 210 46 204 60 165
Polygon -1 true false 75 45 83 77 71 103 86 114 166 78 135 60
Polygon -7500403 true true 30 136 151 77 226 81 280 119 292 146 292 160 287 170 270 195 195 210 151 212 30 166
Circle -16777216 true false 215 106 30

flag
false
0
Rectangle -7500403 true true 60 15 75 300
Polygon -7500403 true true 90 150 270 90 90 30
Line -7500403 true 75 135 90 135
Line -7500403 true 75 45 90 45

flower
false
0
Polygon -10899396 true false 135 120 165 165 180 210 180 240 150 300 165 300 195 240 195 195 165 135
Circle -7500403 true true 85 132 38
Circle -7500403 true true 130 147 38
Circle -7500403 true true 192 85 38
Circle -7500403 true true 85 40 38
Circle -7500403 true true 177 40 38
Circle -7500403 true true 177 132 38
Circle -7500403 true true 70 85 38
Circle -7500403 true true 130 25 38
Circle -7500403 true true 96 51 108
Circle -16777216 true false 113 68 74
Polygon -10899396 true false 189 233 219 188 249 173 279 188 234 218
Polygon -10899396 true false 180 255 150 210 105 210 75 240 135 240

house
false
0
Rectangle -7500403 true true 45 120 255 285
Rectangle -16777216 true false 120 210 180 285
Polygon -7500403 true true 15 120 150 15 285 120
Line -16777216 false 30 120 270 120

leaf
false
0
Polygon -7500403 true true 150 210 135 195 120 210 60 210 30 195 60 180 60 165 15 135 30 120 15 105 40 104 45 90 60 90 90 105 105 120 120 120 105 60 120 60 135 30 150 15 165 30 180 60 195 60 180 120 195 120 210 105 240 90 255 90 263 104 285 105 270 120 285 135 240 165 240 180 270 195 240 210 180 210 165 195
Polygon -7500403 true true 135 195 135 240 120 255 105 255 105 285 135 285 165 240 165 195

line
true
0
Line -7500403 true 150 0 150 300

line half
true
0
Line -7500403 true 150 0 150 150

pentagon
false
0
Polygon -7500403 true true 150 15 15 120 60 285 240 285 285 120

person
false
0
Circle -7500403 true true 110 5 80
Polygon -7500403 true true 105 90 120 195 90 285 105 300 135 300 150 225 165 300 195 300 210 285 180 195 195 90
Rectangle -7500403 true true 127 79 172 94
Polygon -7500403 true true 195 90 240 150 225 180 165 105
Polygon -7500403 true true 105 90 60 150 75 180 135 105

plant
false
0
Rectangle -7500403 true true 135 90 165 300
Polygon -7500403 true true 135 255 90 210 45 195 75 255 135 285
Polygon -7500403 true true 165 255 210 210 255 195 225 255 165 285
Polygon -7500403 true true 135 180 90 135 45 120 75 180 135 210
Polygon -7500403 true true 165 180 165 210 225 180 255 120 210 135
Polygon -7500403 true true 135 105 90 60 45 45 75 105 135 135
Polygon -7500403 true true 165 105 165 135 225 105 255 45 210 60
Polygon -7500403 true true 135 90 120 45 150 15 180 45 165 90

sheep
false
15
Circle -1 true true 203 65 88
Circle -1 true true 70 65 162
Circle -1 true true 150 105 120
Polygon -7500403 true false 218 120 240 165 255 165 278 120
Circle -7500403 true false 214 72 67
Rectangle -1 true true 164 223 179 298
Polygon -1 true true 45 285 30 285 30 240 15 195 45 210
Circle -1 true true 3 83 150
Rectangle -1 true true 65 221 80 296
Polygon -1 true true 195 285 210 285 210 240 240 210 195 210
Polygon -7500403 true false 276 85 285 105 302 99 294 83
Polygon -7500403 true false 219 85 210 105 193 99 201 83

square
false
0
Rectangle -7500403 true true 30 30 270 270

square 2
false
0
Rectangle -7500403 true true 30 30 270 270
Rectangle -16777216 true false 60 60 240 240

star
false
0
Polygon -7500403 true true 151 1 185 108 298 108 207 175 242 282 151 216 59 282 94 175 3 108 116 108

target
false
0
Circle -7500403 true true 0 0 300
Circle -16777216 true false 30 30 240
Circle -7500403 true true 60 60 180
Circle -16777216 true false 90 90 120
Circle -7500403 true true 120 120 60

tree
false
0
Circle -7500403 true true 118 3 94
Rectangle -6459832 true false 120 195 180 300
Circle -7500403 true true 65 21 108
Circle -7500403 true true 116 41 127
Circle -7500403 true true 45 90 120
Circle -7500403 true true 104 74 152

triangle
false
0
Polygon -7500403 true true 150 30 15 255 285 255

triangle 2
false
0
Polygon -7500403 true true 150 30 15 255 285 255
Polygon -16777216 true false 151 99 225 223 75 224

truck
false
0
Rectangle -7500403 true true 4 45 195 187
Polygon -7500403 true true 296 193 296 150 259 134 244 104 208 104 207 194
Rectangle -1 true false 195 60 195 105
Polygon -16777216 true false 238 112 252 141 219 141 218 112
Circle -16777216 true false 234 174 42
Rectangle -7500403 true true 181 185 214 194
Circle -16777216 true false 144 174 42
Circle -16777216 true false 24 174 42
Circle -7500403 false true 24 174 42
Circle -7500403 false true 144 174 42
Circle -7500403 false true 234 174 42

turtle
true
0
Polygon -10899396 true false 215 204 240 233 246 254 228 266 215 252 193 210
Polygon -10899396 true false 195 90 225 75 245 75 260 89 269 108 261 124 240 105 225 105 210 105
Polygon -10899396 true false 105 90 75 75 55 75 40 89 31 108 39 124 60 105 75 105 90 105
Polygon -10899396 true false 132 85 134 64 107 51 108 17 150 2 192 18 192 52 169 65 172 87
Polygon -10899396 true false 85 204 60 233 54 254 72 266 85 252 107 210
Polygon -7500403 true true 119 75 179 75 209 101 224 135 220 225 175 261 128 261 81 224 74 135 88 99

wheel
false
0
Circle -7500403 true true 3 3 294
Circle -16777216 true false 30 30 240
Line -7500403 true 150 285 150 15
Line -7500403 true 15 150 285 150
Circle -7500403 true true 120 120 60
Line -7500403 true 216 40 79 269
Line -7500403 true 40 84 269 221
Line -7500403 true 40 216 269 79
Line -7500403 true 84 40 221 269

wolf
false
0
Polygon -16777216 true false 253 133 245 131 245 133
Polygon -7500403 true true 2 194 13 197 30 191 38 193 38 205 20 226 20 257 27 265 38 266 40 260 31 253 31 230 60 206 68 198 75 209 66 228 65 243 82 261 84 268 100 267 103 261 77 239 79 231 100 207 98 196 119 201 143 202 160 195 166 210 172 213 173 238 167 251 160 248 154 265 169 264 178 247 186 240 198 260 200 271 217 271 219 262 207 258 195 230 192 198 210 184 227 164 242 144 259 145 284 151 277 141 293 140 299 134 297 127 273 119 270 105
Polygon -7500403 true true -1 195 14 180 36 166 40 153 53 140 82 131 134 133 159 126 188 115 227 108 236 102 238 98 268 86 269 92 281 87 269 103 269 113

x
false
0
Polygon -7500403 true true 270 75 225 30 30 225 75 270
Polygon -7500403 true true 30 75 75 30 270 225 225 270

@#$#@#$#@
NetLogo 5.0.5
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
@#$#@#$#@
default
0.0
-0.2 0 0.0 1.0
0.0 1 1.0 0.0
0.2 0 0.0 1.0
link direction
true
0
Line -7500403 true 150 150 90 180
Line -7500403 true 150 150 210 180

@#$#@#$#@
0
@#$#@#$#@
